<?php

abstract class ProgressDialogManager
{
    public function close_allowed() {
        return true;
    }
    public function on_close($user_input) {
        return null;
    }
    public function on_event($ev, $user_input) {
        return null;
    }
    public function on_timer($user_input) {
        return null;
    }
}

class ProgressDialogLabel
{
    public $caption;
    public $halign;
    public $color;

    public function __construct($caption, $halign=null, $color=null) {
        $this->caption = $caption;
        $this->halign = $halign;
        $this->color = $color;
    }
}

class ProgressDialog implements UserInputHandler
{
    const DEBUG = 0;

    public $id;

    public $timer_delay = 4900;

    public $witdh = 0;
    public $never_exit_playback = 0;

    public $labels;
    public $events;
    public $control_ids;
    public $manager;

    public $opened = 0;

    private $controls_dirty = false;
    private $behaviour_dirty = false;

    private $start_action = null;

    public function __construct($id) {
        $this->id = $id;

        $this->labels = array();
        $this->events = array();
        $this->control_ids = array();
        $this->manager = null;

        UserInputHandlerRegistry::get_instance()->register_handler($this);
    }

    public function get_handler_id() {
        return $this->id;
    }

    public function custom_action($control_id, $params=null) {
        return UserInputHandlerRegistry::create_action($this,
            $control_id, $params);
    }

    ///////////////////////////////////////////////////////////////////////

    public function set_manager($manager) {
        if ($this->manager != $manager) {
            $this->set_controls_dirty();
            $this->set_behaviour_dirty();
        }
        $this->manager = $manager;
    }

    public function set_timer_delay($timer_delay) {
        $this->timer_delay = $timer_delay;
        $this->behaviour_dirty = true;
    }

    public function set_dialog_params($width, $never_exit_playback=0) {
        $this->width = $width;
        $this->never_exit_playback = $never_exit_playback;
    }

    public function set_labels($labels) {
        $this->labels = is_array($labels) ? $labels : array();
        $this->set_controls_dirty();
    }

    public function set_events($events, $control_ids) {
        $this->events = is_array($events) ? $events : array();
        $this->control_ids = is_array($control_ids) ? $control_ids : array();
        $this->behaviour_dirty = true;
    }

    public function set_controls_dirty() {
        $this->controls_dirty = true;
    }

    public function set_behaviour_dirty() {
        $this->behaviour_dirty = true;
    }

    ///////////////////////////////////////////////////////////////////////

    public function open($start_action=null)
    {
        if ($this->opened) {
            hd_print("$this->id W: already opened");
            return $start_action;
        }

        hd_print("$this->id I: now opened");

        $this->opened = 1;
        $this->start_action = $start_action;

        $attrs = array(
            'actions' => $this->get_actions(),
            'timer' => $this->get_timer(),
        );
        $params = array(
            'frame_style' => DIALOG_FRAME_STYLE_PLAYBACK_INFO,
            'darkens_screen' => 0);
        if ($this->never_exit_playback)
            $params['never_exit_playback'] = 1;
        $attrs['params'] = $params;
        return ActionFactory::show_dialog(null,
            $this->get_defs(), false, $this->width, $attrs);
    }

    public function close_and_run($user_input, $action=null)
    {
        if (!$this->opened) {
            hd_print("$this->id W: not opened");
            return $action;
        }

        hd_print("$this->id I: closing");

        if ($this->manager)
            $this->manager->on_close($user_input);
        $this->opened = 0;
        return ActionFactory::close_dialog_and_run($action);
    }

    ///////////////////////////////////////////////////////////////////////

    public function get_defs() {
        $defs = array();
        ControlFactory::add_vgap($defs, -8);

        foreach ($this->labels as $obj) {
            $gui_params = null;
            if (isset($obj->halign))
                $gui_params['halign'] = $obj->halign;
            if (isset($obj->color))
                $gui_params['color'] = $obj->color;
            ControlFactory::add_label($defs, null, $obj->caption, $gui_params);
        }

        $this->controls_dirty = false;
        return $defs;
    }

    public function get_timer() {
        if ($this->start_action)
            return ActionFactory::timer(0);
        return $this->timer_delay < 0 ? null :
            ActionFactory::timer($this->timer_delay);
    }

    public function get_actions() {
        $actions = array(
            GUI_EVENT_KEY_RETURN => $this->custom_action('close'));
        if ($this->timer_delay >= 0)
            $actions[GUI_EVENT_TIMER] = $this->custom_action('timer');
        foreach ($this->events as $ev)
            $actions[$ev] = $this->custom_action($ev);
        return $actions;
    }

    ///////////////////////////////////////////////////////////////////////

    public function reset_controls($post_action=null, $initial_sel_ndx=-1) {
        return ActionFactory::reset_controls(
            $this->get_defs(), $post_action, $initial_sel_ndx);
    }

    public function change_behaviour($post_action=null) {
        $this->behaviour_dirty = false;
        return ActionFactory::change_behaviour(
            $this->get_actions(), $this->get_timer(), $post_action);
    }

    ///////////////////////////////////////////////////////////////////////

    public function handle_user_input(&$user_input, &$plugin_cookies)
    {
        $control_id = HD::get($user_input, 'control_id', 'none');
        if (self::DEBUG) {
            if ($control_id != GUI_EVENT_TIMER) {
                hd_print("$this->id: handle_user_input");
                foreach ($user_input as $key => $value)
                    hd_print("  $key => $value");
            }
        } else {
            hd_print("$this->id: user input: $control_id");
        }

        if (!$this->opened) {
            if ($control_id == 'close') {
                // This should not happen normally; workaround allowing
                // user to force-close dialog if stuck again.
                return ActionFactory::close_dialog();
            }

            hd_print("$this->id: not opened (ignored)");
            return null;
        }

        if ($control_id == 'close') {
            if ($this->manager && !$this->manager->close_allowed())
                return null;
            MainHelper::$pb_stat->set_switch_reason('');
            return $this->close_and_run($user_input);
        }

        if ($control_id == 'timer') {
            if ($this->start_action) {
                $a = $this->start_action;
                $this->start_action = null;
                return $this->change_behaviour($a);
            }

            $a = $this->manager ?
                $this->manager->on_timer($user_input) : null;
            if ($this->controls_dirty)
                $a = $this->reset_controls($a);
            return $this->change_behaviour($a);
        }

        foreach (array_merge($this->events, $this->control_ids) as $ev) {
            if ($this->manager && $ev == $control_id) {
                $a = $this->manager->on_event($ev, $user_input);
                if ($this->controls_dirty)
                    $a = $this->reset_controls($a);
                if ($this->behaviour_dirty)
                    $a = $this->change_behaviour($a);
                return $a;
            }
        }

        return null;
    }
}

?>
